uses crt,graph;

const AbschnittsLaenge      =    7.5;
      MaxStrassenAbschnitte =   1000;
      MaxSpuren             =      3;
      MaxNormalTempo        =      5;
      MaxUeberholTempo      =      6;
      Vorwaerts             =      1;
      Rueckwaerts           =     -1;

type tAbschnitt = record
                     Benutzt         : boolean;
                     Geschwindigkeit : byte;
                  end;
type tStrasse   = array[1..MaxSpuren,-MaxUeberholTempo..MaxStrassenAbschnitte+MaxUeberholTempo] of tAbschnitt;

var Strasse          : tStrasse;
    Troedler         : real;
    Verkehrsdichte   : real;
    Nachrueckdichte  : real;
    StrassenLaenge   : real;
    AnzahlAbschnitte : longint;
    AnzahlSpuren     : longint;
    MaxTempo         : array[1..MaxSpuren] of byte;

    FlipPages        : byte;


function ErmittleAbstand(Spur,Abschnitt: longint; Richtung: longint): byte;
var Abstand  : byte;
begin
   Abstand:=0;
   repeat
      inc(Abstand);
   until Strasse[Spur,Abschnitt+Richtung*Abstand].Benutzt or (Abstand>MaxTempo[Spur]);
   ErmittleAbstand:=Abstand-1;
end;


procedure ErzeugeVerkehrslage;
var AbschnittsZaehler : longint;
    SpurenZaehler     : longint;
begin
   fillchar(MaxTempo,sizeof(MaxTempo),MaxUeberholTempo);
   MaxTempo[1]:=MaxNormalTempo;

   fillchar(Strasse,sizeof(Strasse),false);

   randomize;
   for SpurenZaehler:=AnzahlSpuren downto 1 do
      for AbschnittsZaehler:=AnzahlAbschnitte downto 1 do
         if random<=Verkehrsdichte then
         begin
            Strasse[SpurenZaehler,AbschnittsZaehler].Geschwindigkeit:=
                  ErmittleAbstand(SpurenZaehler,AbschnittsZaehler,Vorwaerts);
            Strasse[SpurenZaehler,AbschnittsZaehler].Benutzt:=true;
         end
         else
         begin
            Strasse[SpurenZaehler,AbschnittsZaehler].Geschwindigkeit:=0;
            Strasse[SpurenZaehler,AbschnittsZaehler].Benutzt:=false
         end;
end;



procedure ErgaenzeVerkehrslage;
var AbschnittsZaehler : longint;
    SpurenZaehler     : byte;
    VorgaengerAuto    : longint;
begin
   for SpurenZaehler:=AnzahlSpuren downto 1 do
   begin
      VorgaengerAuto:=ErmittleAbstand(SpurenZaehler,1,Vorwaerts);

      if VorgaengerAuto=0 then
         exit;

      for AbschnittsZaehler:=VorgaengerAuto downto 1 do
         if random<=Nachrueckdichte then
         begin
            Strasse[SpurenZaehler,AbschnittsZaehler].Geschwindigkeit:=
                  ErmittleAbstand(SpurenZaehler,AbschnittsZaehler,Vorwaerts);
            Strasse[SpurenZaehler,AbschnittsZaehler].Benutzt:=true;
         end
         else
         begin
            Strasse[SpurenZaehler,AbschnittsZaehler].Geschwindigkeit:=0;
            Strasse[SpurenZaehler,AbschnittsZaehler].Benutzt:=false
         end;
   end;
end;


procedure SimuliereVerkehrslage;
var AbschnittsZaehler : longint;
    NeuerAbschnitt    : longint;
    NeueSpur          : longint;
    SpurenZaehler     : longint;
    Abstand           : byte;
    Geschwindigkeit   : byte;
    Beschleunigt      : boolean;
begin
   for SpurenZaehler:=1 to AnzahlSpuren do
      for AbschnittsZaehler:=AnzahlAbschnitte downto 1 do
         if Strasse[SpurenZaehler,AbschnittsZaehler].Benutzt then
         begin
            Geschwindigkeit:=Strasse[SpurenZaehler,AbschnittsZaehler].Geschwindigkeit;
            Strasse[SpurenZaehler,AbschnittsZaehler].Benutzt:=false;
            Strasse[SpurenZaehler,AbschnittsZaehler].Geschwindigkeit:=0;

            NeuerAbschnitt:=AbschnittsZaehler+Geschwindigkeit;
            if NeuerAbschnitt>AnzahlAbschnitte then
               continue;

            NeueSpur:=SpurenZaehler;
            if NeueSpur>1 then
               if not Strasse[NeueSpur-1,NeuerAbschnitt].Benutzt then
               begin
                  Abstand:=ErmittleAbstand(NeueSpur-1,NeuerAbschnitt,Rueckwaerts);
                  if (Abstand>Strasse[NeueSpur-1,NeuerAbschnitt-Abstand-1].Geschwindigkeit) then
                  begin
                     Abstand:=ErmittleAbstand(NeueSpur-1,NeuerAbschnitt,Vorwaerts);
                     if (Abstand>Geschwindigkeit) then
                        dec(NeueSpur);
                  end;
               end;

            Beschleunigt:=false;
            Abstand:=ErmittleAbstand(NeueSpur,NeuerAbschnitt,Vorwaerts);
            if Abstand>Geschwindigkeit then
            begin
               inc(Geschwindigkeit);
               Beschleunigt:=true;
            end
            else
            begin
               if (Geschwindigkeit<MaxTempo[NeueSpur]) and (NeueSpur<AnzahlSpuren) then
                  if not Strasse[NeueSpur+1,NeuerAbschnitt].Benutzt then
                     if Geschwindigkeit>=Strasse[NeueSpur,NeuerAbschnitt+Abstand+1].Geschwindigkeit+2 then
                        inc(NeueSpur);
               Geschwindigkeit:=ErmittleAbstand(NeueSpur,NeuerAbschnitt,Vorwaerts);
            end;

            if random<=Troedler then
            begin
               if Beschleunigt then
                  dec(Geschwindigkeit);
               if Geschwindigkeit>0 then
                  dec(Geschwindigkeit);
            end;

            if NeuerAbschnitt<=MaxStrassenAbschnitte then
            begin
               Strasse[NeueSpur,NeuerAbschnitt].Benutzt:=true;
               Strasse[NeueSpur,NeuerAbschnitt].Geschwindigkeit:=Geschwindigkeit;
            end;
         end;
end;


procedure ZeigeVerkehrslage;
var AbschnittsZaehler     : longint;
    SpurenZaehler         : longint;
    Autos                 : longint;
    Geschwindigkeitssumme : longint;
    Zeichenkette          : string;
begin
   Autos:=0;
   Geschwindigkeitssumme:=0;

   setactivepage(FlipPages);
   FlipPages:=1-FlipPages;
   setvisualpage(FlipPages);

   setviewport(250,0,350,70,false);
   clearviewport;
   setviewport(0,0,639,349,true);

   for SpurenZaehler:=AnzahlSpuren downto 1 do
   begin
      for AbschnittsZaehler:=1 to AnzahlAbschnitte do
      begin
         putpixel(AbschnittsZaehler,350-Spurenzaehler*50,black);

         if Strasse[SpurenZaehler,AbschnittsZaehler].Benutzt then
         begin
            inc(Autos);
            inc(Geschwindigkeitssumme,Strasse[SpurenZaehler,AbschnittsZaehler].Geschwindigkeit);

            if Strasse[SpurenZaehler,AbschnittsZaehler].Geschwindigkeit>0 then
               putpixel(AbschnittsZaehler,350-Spurenzaehler*50,white)
            else
               putpixel(AbschnittsZaehler,350-Spurenzaehler*50,red);
         end;
      end;
   end;

   setcolor(white);
   str(Autos:5,Zeichenkette);
   outtextxy(0, 0,'Anzahl Autos                 : '+Zeichenkette);
   str(Geschwindigkeitssumme/Autos:8:2,Zeichenkette);
   outtextxy(0,20,'Durchschnittsgeschwindigkeit : '+Zeichenkette);
   str(Geschwindigkeitssumme:5,Zeichenkette);
   outtextxy(0,40,'Geschwindigkeitsindex        : '+Zeichenkette);
   str(100*Autos/(AnzahlAbschnitte*longint(AnzahlSpuren)):8:2,Zeichenkette);
   outtextxy(0,60,'Tatschliche Verkehrsdichte  : '+Zeichenkette+' %');
end;



var Treiber,Modus : integer;
begin
   Verkehrsdichte:=0.2;
   Nachrueckdichte:=0.5;
   Troedler:=0.7;
   StrassenLaenge:=4500;
   AnzahlAbschnitte:=round(StrassenLaenge/AbschnittsLaenge);

   clrscr;
   write('Anzahl Spuren : ');
   readln(AnzahlSpuren);

   ErzeugeVerkehrslage;

   Treiber:=VGA;
   Modus:=VGAMed;
   initgraph(Treiber,Modus,'');
   FlipPages:=0;

   while not keypressed do
   begin
      ErgaenzeVerkehrslage;
      ZeigeVerkehrslage;
      SimuliereVerkehrslage;
      delay(10);
   end;

   closegraph;
end.